package com.perforce.workshop.tjuricek.p4oauth;

import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ExpectedConditions;
import org.testng.annotations.AfterSuite;
import org.testng.annotations.BeforeSuite;
import org.testng.annotations.Test;

import java.net.InetAddress;
import java.net.UnknownHostException;

import static org.openqa.selenium.support.ui.ExpectedConditions.visibilityOfElementLocated;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertTrue;

/**
 */
public class AuthorizationCodeTest implements WebdriverTest {

    private WebDriver webDriver;

    private String accessToken;

    /**
     * Runs a simple positive check using our test server's authorization
     * code workflow.
     */
    @Test
    public void authorizationCodeWorkflow() {
        webDriver.get("https://localhost:8444/start");
        webDriver.findElement(By.id("start-auth")).click();

        // We should be redirected to the p4oauth instance now

        getWait().until(ExpectedConditions.titleIs("Log in to Perforce"));

        WebElement nameEl = webDriver.findElement(By.cssSelector("[name=client_id]"));
        nameEl.clear();
        nameEl.sendKeys("jdoe");

        webDriver.findElement(By.cssSelector("[name=password]"))
                .sendKeys("johndoe1A");

        webDriver.findElement(By.cssSelector("button[type=submit]"))
                .click();

        // We redirect back to the test server, to a page that does a couple
        // of operations.
        //
        // 1. A background POST to p4oauth to get the authorization code
        // 2. A background POST to the test server to list files locally
        //
        //

        getWait().until(visibilityOfElementLocated(By.cssSelector("#results")));
        String paths = webDriver.findElement(By.cssSelector("#results")).getText();
        assertTrue(paths.contains("doh"));

        accessToken = webDriver.findElement(By.id("access-token")).getText().trim();
        assertNotNull(accessToken);
    }

    @Test(dependsOnMethods = "authorizationCodeWorkflow")
    public void fetchP4TokenAfterAuthorizationCode() {

        webDriver.get("https://localhost:8444/fetcher?client_id=jdoe&access_token=" + accessToken);

        // Again, we'll wait for the handler page to be called and list files
        // locally, but, there will be no access token returned

        getWait().until(visibilityOfElementLocated(By.cssSelector("#results")));
        String paths = webDriver.findElement(By.cssSelector("#results")).getText();
        assertTrue(paths.contains("doh"));
    }

    @BeforeSuite
    public void setup() {
        initWebdriver();
    }


    public WebDriver getWebDriver() {
        return webDriver;
    }

    public void setWebDriver(WebDriver webDriver) {
        this.webDriver = webDriver;
    }

    @AfterSuite
    public void cleanup() {
        stopWebDriver();
    }
}
